/**
 * Predefined set of features that enable/disable certain application features and plugins.
 * - `Full`: All features of the application are enabled. Typically used to create/edit complete article lists or quotes.
 *    Can also be used in case a customized feature set is desired. In this case additional user restrictions and disabled plugins can be configured.
 * - `Configurator`: Provides a reduced UI suitable for article/product configuration by disabling all features/plugins.
 *    All features not necessary for product configuration are disabled (e.g. ordering, reporting or calculation).
 */
export type FeatureSet = "Full" | "Configurator";

/**
 * Mode for the project editor.
 * - `Default`:  Used to create and edit a project.
 * - `Report`:  Used to send/print a report.
 * - `Order`:  Used to place an order.
 */
export type ProjectEditorMode = "Default" | "Report" | "Order";

/** Default settings for an eaiws session. */
export type DefaultSessionSetting = "PartyInCharge" | "Currency" | "ExchangeRates" | "TaxScheme" | "DataLanguage" | "ProjectValidity";

export interface ProjectMetaData
{
    /** Sales group (SGR) as defined in the OEX specification. */
    salesGroup?: string;
    /** Sales organization (SOR) as defined in the specification. */
    salesOrganization?: string;
    /** Predefined order number of the project */
    orderNumber?: string;
    /** Predefined vendor number of the project */
    vendorNumber?: string;
}

export interface ButtonOptions
{
    /** custom tooltip for the button */
    tooltip?: string;
    /**
     * custom icon name for the button (ignored if `iconUrl` is specified)
     * See `https://materialdesignicons.com/` for list of icons.
     * Icon names have to be prefixed with `mdi-`.
     */
    iconName?: string;
    /** custom icon url for the button */
    iconUrl?: string;
}

/** `wbk.configuration` message parameter. */
export interface WbkConfiguration
{
    /** eaiws related configurations */
    eaiws: {
        /** base url of the EAIWS server */
        baseUrl: string;
        /** id of the EAIWS session */
        sessionId: string;
        /** interval for the session keep alive message in milliseconds, 0 disables keep alive (default: 60000) */
        keepAliveInterval?: number;
    };

    /** application related configurations */
    application?: {
        /**
         * Predefined set of features that enable/disable certain application features and plugins. (default: `Full`)
         */
        featureSet?: FeatureSet;
        /**
         * If defined, overrides the users default language used for the product data (array of langauges in order of priority).
         * If the array is empty the UI language will be used as fallback.
         * The default data language is only used if `project.applySessionDefaults` is specified.
         * Otherwise the language which is currently configured in the eaiws session will be used.
         */
        dataLanguage?: Array<string>;

        /**
         * Enables/disables article price date support (default: false)
         * Note: To correctly load/preserve the price date stored in OBK/OBX files it is importent to set the
         * session property `egr.eai.basket.preserve_price_date` of the EAIWS session to `true` before loading the OBK/OBX.
         */
        priceDateSupported?: boolean;

        /**
         * Theme settings which will override the default application theme.
         */
        theme?: {
            /** Primary color of the theme as hex string. (default: #0078d4) */
            primaryColor?: string;
            /** Text color of the theme as hex string. (default: #333333) */
            textColor?: string;
            /** Application bar color of the theme as hex string. (default: #ffffff) */
            appBarColor?: string;
            /**
             * Logo of the application. (default: null)
             */
            logo?: string | null;
            /**
             * Background color of the logo as hex/css color string. (default: transparent)
             */
            logoBackgroundColor?: string;
        };

        /**
         * Optional list of plugins (plugin names) that will be disabled.
         */
        disabledPlugins?: Array<string>;
        /**
         * Optional list of plugins categories that will be disabled.
         */
        disabledPluginCategories?: Array<string>;
        /** If true and `user` is defined the application will show the user information. (default: false)*/
        showUser?: boolean;
        /**
         * If true the application will show a `Done` button which can be used
         * from the user to finish the integration. (default: true)
         */
        showDoneButton?: boolean;

        /** Additional options for the done button. */
        doneButtonOptions?: ButtonOptions;

        /**
         * If true the application will show a `Save` button which can be used
         * from the user to trigger the saving of the current project. (default: false)
         * The message `wbkHost.saveProject` will be send to the host after clicking the button.
         * The host should then save the project. The basket will be locked until the host sends
         * the message `wbk.saveProjectDone` back to the basket.
         */
        showProjectSaveButton?: boolean;
    };

    /** user related configurations */
    user?: {
        /** Login name of the user. */
        name: string;
        /** Optional full name of the user (e.g. first name + last name) */
        fullName?: string;
        /** Optional url to the company logo. */
        companyLogo?: string;
        /** Optional url for user account management */
        accountUrl?: string;
        /**
         * Optional set of restrictions for this user. If not defined the
         * default restrictions of the application will be used.
         * See basket integration manual for a list of available restrictions.
         */
        restrictions?: Array<string>;
        /**
         * Optional pCon.login access token.
         * Some basket operations are only available for logged in users
         * e.g. accessing protected PIM article information or to access user/organization preferences
         * Note: The access token usually expires after 1 hour. Its important to request a new token before that happens and send it
         * to the basket using the `wbk.updateUserAccessToken` message.
         */
        accessToken?: string;
    };

    /** Configuration for the project and item calculation. */
    calculation?: {
        /** name of the pricing procedure to use for the project and item calculation */
        pricingProcedure?: string;
    };

    /** project related configurations */
    project?: {
        /** initial title */
        title?: string;
        /** if true the title may be changed by the user */
        titleEditable?: boolean;
        /**
         * The editor mode defines the primary purpose of the project editor. If not defined `Default`
         * will be used.
         */
        editorMode?: ProjectEditorMode;
        /**
         *  If `true` all of the current default session settings will be assign to the session.
         *  Can be used to initialize a new project with the user defaults.
         *  An array with `DefaultSessionSetting` entries can be used to define which settings will be applied.
         */
        applySessionDefaults?: boolean | Array<DefaultSessionSetting>;
        /** Optional list of locked items which will be used to lock certain item related operations. */
        lockedItems?: Array<LockedItem>;
        /**
         * If true, legacy data (e.g. header or address data) will be loaded and migrated. (default: false)
         * Should be used when loading obk files of older applications.
         */
        migrateLegacyProjectData?: boolean;
        /** Controls if the catalog should be opened automatically for empty/new projects. (default: false) */
        openCatalogForEmptyProjects: boolean;
        /**
         * Optional project related meta data which may be used by certain operations (e.g. ordering)
         */
        metaData?: ProjectMetaData;
    };

    /** Configuration for the reporting. */
    report?: {
        /**
         * Array of template names of report templates which should be available to the user.
         */
        templates?: Array<string>;

        /** Name of the default template */
        defaultTemplate?: string;

        /**
         * Predefined attachments for each template which will be added to every report.
         * The name of the template is used as the key for the dictionary.
         * If an empty string is used as the template name the provided attachments
         * will be applied to all templates without explicitly defined attachments.
         */
        attachments?: {
            [template: string]: {
                /** List of PDF urls which should be added in front of the report. */
                prependUrls?: Array<string>;
                /** List of PDF urls which should be added at the end of the report. */
                appendUrls?: Array<string>;
            };
        };
    };

    /** Configuration for the catalog. */
    catalog?: {
        /** Custom catalog registration */
        customCatalogs?: Array<CustomCatalog>;
    };

    /** Ordering configuration. */
    order?: {
        /**
         * If defined a report using the specified template will
         * be generated and attached to the order. (default: null)
         */
        reportTemplate: string | null;
    };
}

export interface LockedItem
{
    /** id of the locked item */
    itemId: string;
    /**
     * List of locked operations for this item stored in a bitmask.
     * Available flags:
     * - 0x00000000 - nothing is locked
     * - 0x00000001 - Delete: locks the delete operation => item can't be deleted
     * - 0x00000002 - Configuration: locks the configuration of an ofml article => article can't be configured
     * - 0x00000004 - Calculation: locks the calculation of an item => item can't be calculated
     * - 0x00000008 - ItemProperties: locks the item properties => properties like texts or article number can't be changed
     * - 0x00000010 - Image: locks the item image => the image of an item can't be changed
     * - 0x00000020 - Quantity: locks the article quantity => the quantity of an article can't be changed
     * - 0x00000040 - OfmlArticleUpdate: locks the ofml article update => updating an ofml article is not possible
     * - 0xFFFFFFFF - locks all operations
     */
    lockFlags: number;
}

/** Base custom catalog item */
export interface CustomCatalogItem
{
    id: string;
    label: string | { [language: string]: string };
    imageUrl?: string;
}

/** Definition of a custom catalog. */
export interface CustomCatalog extends CustomCatalogItem
{
    /**
     * If provided the catalog will be handled as an external catalog.
     * It will be embedded in the ui using the `externalUrl`.
     * More details can be found in the external catalogs documentation.
     */
    externalUrl?: string;
}

/** Definition of a custom catalog article. Used as parameter in the `wbk.customCatalogArticles` message. */
export interface CustomCatalogArticle extends CustomCatalogItem
{
    catalogId: string;
    articleNumber?: string;
    manufacturerId?: string;
    seriesId?: string;
}

/** Full article data of a custom article. Used as parameter in the `wbk.customArticle` message. */
export interface CustomArticle
{
    manufacturerId?: string;
    seriesId?: string;
    baseArticleNumber?: string;
    finalArticleNumber?: string;
    variantCode?: string;

    /**
     * Quantity of the article
     * @defaultValue 1
     */
    quantity?: number;

    /** short text as simple string or as multi language map (language code -> text) */
    shortText?: string | { [language: string]: string };
    /** long text as simple string or as multi language map (language code -> text) */
    longText?: string | { [language: string]: string };
    /** feature text as simple string or as multi language map (language code -> text) */
    featureText?: string | { [language: string]: string };

    /** Additional comments/notes for this article. */
    comments?: string;

    purchasePrice?: Money;
    salesPrice?: Money;
    packagingInfo?: PackagingInfo;

    /** Map from tax type to tax category for the active EAIWS tax scheme. */
    taxes?: { [type: string]: string };

    /**
     * Url or data-URI for the article image.
     * Currently only jpg and png images are supported.
     */
    imageUrl?: string;
}

export interface Money
{
    value: number;
    currency: string;
}

export interface Value
{
    value: number;
    unit: string;
}

export interface PackagingInfo
{
    width?: Value;
    height?: Value;
    depth?: Value;
    volume?: Value;
    tareWeight?: Value;
    netWeight?: Value;
    itemsPerPackUnit?: number;
    packUnitsPerArticle?: number;
}