import { WbkConfigurationMessage } from "../../libs/integration-api/WbkMessages";

import { Application } from "../Application";
import { GATEKEEPER_ID, PRICING_PROCEDURE_NAME, REPORT_TEMPLATE } from "../Config";
import { getCustomCatalogConfiguration } from "./CustomCatalog";
import { getSessionFromGatekeeper } from "./Gatekeeper";
import { setExampleCustomerAddress, setExampleProjectData } from "./HeaderData";
import { loadOBK, loadOBX } from "./Import";
import { setupPriceConditions } from "./PriceConditions";

/**
 * This function prepares the configuration of pCon.basket including the EAIWS session and sends it to the basket.
 */
export async function prepareAndSendConfiguration(pApp: Application): Promise<void>
{
    //option1: create EAIWS session using gatekeeper. see gatekeeper documentation: https://eaiws-server.pcon-solutions.com/doc/v3
    await getSessionFromGatekeeper(pApp.session,
        GATEKEEPER_ID, pApp.loginHandler.accessToken);

    //option2: create EAIWS session using eaiws baseurl and startup (not supported in pCon.cloud)
    //await pApp.session.open("https://www.example-eaiws.com", {startup: example-startup"});

    //prepare/intialize the session
    const tApplySessionDefault = await preareEaiwsSession(pApp);

    //build the configuration message and send it to pCon.basket
    let tMessage: WbkConfigurationMessage = {
        type: "wbk.configuration",
        parameter: {
            application: {
                theme: { //use custom application colors
                    primaryColor: "#00A000",
                    appBarColor: "#505050"
                },
                //featureSet: "Configurator", //enable configurator mode
                showUser: true,
                showProjectSaveButton: true //see handling of `wbkHost.saveProject` message
            },
            user: {
                /* The pCon.login access token should be provided here, to activate all application features.
                 * e.g. accessing protected PIM article information or to access user/organization preferences
                 * Note: The access token usually expires after 1 hour. Its important to request a new token before that happens and send it
                 * to the basket using the `wbk.updateUserAccessToken` message.
                 */
                accessToken: pApp.loginHandler.accessToken,
                name: "user@example.com",
                fullName: "Example User"
            },
            eaiws: {
                baseUrl: pApp.session.baseUrl,
                sessionId: pApp.session.session.sessionId,
                keepAliveInterval: 0, //disable keep alive, its already handled on our side
            },
            calculation: {
                pricingProcedure: PRICING_PROCEDURE_NAME //use different pricing procedure (optional)
            },
            project: {
                title: "test project",
                titleEditable: true,
                applySessionDefaults: tApplySessionDefault, //start new project with defaults
                openCatalogForEmptyProjects: true //start in catalog
            },
            report: {
                templates: [REPORT_TEMPLATE],
                defaultTemplate: REPORT_TEMPLATE
            },
            catalog: {
                customCatalogs: getCustomCatalogConfiguration()
            }
        }
    }
    pApp.sendMessageToBasket(tMessage);
}

/**
 * This function is used to initailize the EAIWS session.
 */
async function preareEaiwsSession(pApp: Application): Promise<boolean>
{
    //get selected file
    let tInputField = document.getElementById("fileInput") as HTMLInputElement;
    let tSelectedFile = tInputField.files != null ? tInputField.files[0] : null;
    let tApplySessionDefaults = true;

    if (tSelectedFile != null && (tSelectedFile.name).toLowerCase().endsWith(".obk")) {
        //OBK selected -> load OBK
        await loadOBK(pApp.session, tSelectedFile);
        tApplySessionDefaults = false; //do not apply session defaults, keep settings stored in obk
    } else if (tSelectedFile != null && (tSelectedFile.name).toLowerCase().endsWith(".obx")) {
        //OBX selected -> load OBX

        /*
        It is necessary to activate the price calculation before loading an OBX in order to load the discounts from the OBX.
        This step can be removed if the basket calculation is not used.
        */
        await pApp.session.basket.addPriceCalculation(PRICING_PROCEDURE_NAME);

        await loadOBX(pApp.session, tSelectedFile);
    } else {
        //no file -> create new/empty project with prefilled header data
        await setExampleProjectData(pApp.session);
        await setExampleCustomerAddress(pApp.session);
    }

    //setup our price conditions
    await setupPriceConditions(pApp.session);
    return (tApplySessionDefaults);
}